/* (C) 2022 YiRing, Inc. */
package com.yiring.common.web;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.yiring.common.config.MinioConfig;
import com.yiring.common.core.Minio;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.exception.FailStatusException;
import com.yiring.common.param.DownloadParam;
import com.yiring.common.util.FileUtils;
import io.minio.GetObjectResponse;
import io.minio.ObjectWriteResponse;
import io.minio.StatObjectResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * Minio S3
 * @author Jim
 */

@Slf4j
@Validated
@SuppressWarnings({ "deprecation" })
@ApiSupport(order = -98)
@Api(tags = "文件管理", description = "file")
@RequiredArgsConstructor
@RestController
@RequestMapping("/common/file/")
public class MinioController {

    final Minio minio;
    final MinioConfig minioConfig;

    /**
     * minio 上传文件，成功返回文件 url
     */
    @ApiOperation(value = "文件上传", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @PostMapping(value = "upload", headers = HttpHeaders.CONTENT_TYPE + "=" + MediaType.MULTIPART_FORM_DATA_VALUE)
    public Result<String> upload(@ApiParam(value = "文件", required = true) @RequestPart("file") MultipartFile file) {
        try {
            Snowflake snowflake = IdUtil.getSnowflake(1, 1);
            long uuid = snowflake.nextId();

            // 文件上传
            String date = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy/M/d"));
            String folder = "upload/" + date + "/" + uuid;
            ObjectWriteResponse response = minio.putObject(file, folder);
            String uri = minio.getURI(response.object(), minioConfig.getBucket());
            return Result.ok(uri);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.no(Status.BAD_REQUEST, "上传失败");
        }
    }

    /**
     * MinIO 文件下载（非公开桶）
     * @param response HttpServletResponse
     * @param param 请求参数
     */
    @ApiOperation(value = "文件下载", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    @GetMapping("download")
    public void download(HttpServletResponse response, @Valid DownloadParam param) {
        try {
            StatObjectResponse statObject = minio.objectStat(param.getBucket(), param.getObject());
            GetObjectResponse object = minio.getObject(param.getBucket(), param.getObject());
            long lastModified = Timestamp.from(statObject.lastModified().toInstant()).getTime();
            FileUtils.download(
                response,
                object,
                statObject.size(),
                FileUtil.getName(statObject.object()),
                statObject.contentType(),
                lastModified
            );
        } catch (Exception e) {
            throw new FailStatusException(Status.BAD_REQUEST, e.getMessage());
        }
    }
}
